import logging
from functools import lru_cache
from typing import Iterable, Tuple

from fetchfox import rest
from fetchfox.checks import check_str

logger = logging.getLogger(__name__)
BASE_URL = "https://api.nftexplorer.app"


def get(service: str, api_key: str, params: dict = None, version: int = 1) -> Tuple[dict, int]:
    check_str(api_key, "nftexplorerapp.api_key")

    return rest.get(
        url=f"{BASE_URL}/v{version}/{service}",
        headers={
            "Authorization": api_key,
        },
        params=params or {},
        sleep=1,
    )


@lru_cache(maxsize=None)
def get_collection_id(creator_address: str, api_key: str) -> str:
    check_str(creator_address, "randswap.creator_address")
    creator_address = creator_address.strip().upper()

    response, status_code = get(
        "collections/search",
        params={
            "q": creator_address,
        },
        api_key=api_key,
    )

    if response.get("error"):
        logger.error("error fetching sales on algorand: %s", response["error"])
        return

    verified = response["results"].get("verified", [])

    if verified:
        return verified[0]["collectionId"]

    recognized = response["results"].get("recognized", [])

    if recognized:
        return recognized[0]["collectionId"]

    return None


def get_sales(creator_address: str, api_key: str) -> Iterable[dict]:
    check_str(creator_address, "randswap.creator_address")
    creator_address = creator_address.strip().upper()

    collection_id = get_collection_id(creator_address, api_key)

    if not collection_id:
        return

    next_token = ""

    while True:
        response, status_code = get(
            "/collections/salesHistory",
            params={
                "collectionId": collection_id,
                "nextToken": next_token,
            },
            api_key=api_key,
        )

        if response.get("error"):
            logger.error("error fetching sales on algorand: %s", response["error"])
            return

        yield from response["sales"]

        next_token = response.get("nextToken")

        if not next_token:
            break



import json
from base64 import b64decode
from functools import lru_cache
from typing import Iterable, Tuple

from fetchfox import rest
from fetchfox.checks import check_str

BASE_URL = "https://mainnet-idx.algonode.cloud"


def get(service: str, params: dict = None, version: int = 2) -> Tuple[dict, int]:
    return rest.get(
        url=f"{BASE_URL}/v{version}/{service}",
        params=params or {},
    )


def get_assets(creator_address: str) -> Iterable[str]:
    check_str(creator_address, "algonodecloud.creator_address")
    creator_address = creator_address.strip().upper()

    response, status_code = get(f"accounts/{creator_address}")

    assets = sorted(
        response["account"].get("created-assets", []),
        key=lambda a: a["index"],
        reverse=True,
    )

    for asset in assets:
        yield asset["index"]


@lru_cache(maxsize=None)
def get_asset_metadata(asset_id: str) -> dict:
    check_str(asset_id, "algonodecloud.asset_id")

    response, status_code = get(
        f"assets/{asset_id}/transactions",
        params={
            "tx-type": "acfg",
            "limit": "1",
        },
    )

    transaction = response["transactions"][0]
    note = b64decode(transaction["note"]).decode("utf-8")

    return json.loads(note)


@lru_cache(maxsize=None)
def get_asset_data(asset_id: str) -> dict:
    check_str(asset_id, "algonodecloud.asset_id")

    response, status_code = get(f"assets/{asset_id}")

    return response["asset"]["params"]


def get_holdings(address: str) -> Iterable[dict]:
    check_str(address, "algonodecloud.address")
    address = address.strip().upper()

    response, status_code = get(f"accounts/{address}")

    account = response["account"]

    for asset in account.get("assets", []):
        yield asset


def get_owner(asset_id: str) -> dict:
    check_str(asset_id, "algonodecloud.asset_id")

    response, status_code = get(
        f"assets/{asset_id}/balances",
        params={
            "currency-greater-than": "0",
        },
    )

    balances = response.get("balances", [])

    if not balances:
        return None

    return {
        "asset_id": asset_id,
        "address": balances[0]["address"],
        "amount": balances[0]["amount"],
    }



from typing import Tuple

from fetchfox.blockchains.utils import check

ADA_HANDLE_REGEX = r"^\$[a-z0-9\-_.]{1,15}$"
SHORT_ADDRESS_REGEX = r"^addr1[0-9a-z]{53}$"
LONG_ADDRESS_REGEX = r"^addr1[0-9a-z]{98}$"
ASSET_ID_REGEX = r"^[a-f0-9]{56}[a-fA-F0-9]+$"
POLICY_ID_REGEX = r"^[a-f0-9]{56}$"
STAKE_ADDRESS_REGEX = r"^stake1[0-9a-z]{53}$"


def is_ada_handle(string: str) -> bool:
    return check(ADA_HANDLE_REGEX, string)


def is_address(string) -> bool:
    if check(SHORT_ADDRESS_REGEX, string):
        return True

    return check(LONG_ADDRESS_REGEX, string)


def is_asset_id(string: str) -> bool:
    return check(ASSET_ID_REGEX, string)


def is_policy_id(string: str) -> bool:
    return check(POLICY_ID_REGEX, string)


def is_stake_address(string: str) -> bool:
    return check(STAKE_ADDRESS_REGEX, string)


def split_asset_id(asset_id: str) -> Tuple[str, str]:
    return asset_id[:56], bytes.fromhex(asset_id[56:]).decode()


def is_wallet(wallet: str) -> bool:
    return is_stake_address(wallet) or is_address(wallet) or is_ada_handle(wallet)
